--
-- ClaasC540
-- This is the specialization for ClaasC540s
-- It reacts both to steerable and attachable events
--
-- @author  Stefan Geiger
-- @date  04/12/08
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

ClaasC540 = {};

function ClaasC540.prerequisitesPresent(specializations)
    return true;
end;

function ClaasC540:load(xmlFile)

	local numSpinnerParts = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.SpinnerParts#count"), 0);
    self.SpinnerParts = {};
    for i=1, numSpinnerParts do
		local partnamei = string.format("vehicle.SpinnerParts.part" .. "%d", i);
		self.SpinnerParts[i] = {};
		self.SpinnerParts[i].index = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, partnamei .. "#index"));
		self.SpinnerParts[i].Scale = Utils.getNoNil(getXMLInt(xmlFile, partnamei .. "#rotScale"), 10);
    end;
	
	local translationPart2Node = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.translationPart2#index"));
    if translationPart2Node ~= nil then
        self.translationPart2 = {};
        self.translationPart2.node = translationPart2Node;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPart2#minTrans"));
        self.translationPart2.minTrans = {};
        self.translationPart2.minTrans[1] = Utils.getNoNil(x, 0);
        self.translationPart2.minTrans[2] = Utils.getNoNil(y, 0);
        self.translationPart2.minTrans[3] = Utils.getNoNil(z, 0);
        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPart2#maxTrans"));
        self.translationPart2.maxTrans = {};
        self.translationPart2.maxTrans[1] = Utils.getNoNil(x, 0);
        self.translationPart2.maxTrans[2] = Utils.getNoNil(y, 0);
        self.translationPart2.maxTrans[3] = Utils.getNoNil(z, 0);
        self.translationPart2.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPart2#transTime"), 2)*1000;
        self.translationPart2.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPart2#touchTransLimit"), 10);
    end;
	
   	local translationPartNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.translationPart#index"));
    if translationPartNode ~= nil then
        self.translationPart = {};
        self.translationPart.node = translationPartNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPart#minTrans"));
        self.translationPart.minTrans = {};
        self.translationPart.minTrans[1] = Utils.getNoNil(x, 0);
        self.translationPart.minTrans[2] = Utils.getNoNil(y, 0);
        self.translationPart.minTrans[3] = Utils.getNoNil(z, 0);
        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPart#maxTrans"));
        self.translationPart.maxTrans = {};
        self.translationPart.maxTrans[1] = Utils.getNoNil(x, 0);
        self.translationPart.maxTrans[2] = Utils.getNoNil(y, 0);
        self.translationPart.maxTrans[3] = Utils.getNoNil(z, 0);
        self.translationPart.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPart#transTime"), 2)*1000;
		self.translationPart.touchTransLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPart#touchTransLimit"), 10));
    end;

	local numHaspelParts = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.HaspelParts#count"), 0);
    self.HaspelParts = {};
    for i=1, numHaspelParts do
      local partnamei = string.format("vehicle.HaspelParts.part" .. "%d", i);
      self.HaspelParts[i] = {};
      self.HaspelParts[i].index = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, partnamei .. "#index"));
      local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partnamei .. "#minRot"));
      self.HaspelParts[i].minRot = {};
      self.HaspelParts[i].minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
      self.HaspelParts[i].minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
      self.HaspelParts[i].minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
      x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partnamei .. "#maxRot"));
      self.HaspelParts[i].maxRot = {};
      self.HaspelParts[i].maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
      self.HaspelParts[i].maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
      self.HaspelParts[i].maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
      self.HaspelParts[i].rotTime = Utils.getNoNil(getXMLString(xmlFile,  partnamei .. "#rotSpeed"), 2)*1000;
    end;
	
	self.numhideWarn = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.hideWarn#count"), 0);
    self.hideWarn = {};
    for i=1, self.numhideWarn do
        local hideWarnnamei = string.format("vehicle.hideWarn.hideWarn" .. "%d", i);
        self.hideWarn[i] = Utils.indexToObject(self.rootNode, getXMLInt(xmlFile, hideWarnnamei .. "#index"));
    end;
	
    local numRotationParts = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.rotationParts#count"), 0);
    self.rotationParts = {};
    for i=1, numRotationParts do
      local partnamei = string.format("vehicle.rotationParts.part" .. "%d", i);
      self.rotationParts[i] = {};
      self.rotationParts[i].index = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, partnamei .. "#index"));
      local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partnamei .. "#minRot"));
      self.rotationParts[i].minRot = {};
      self.rotationParts[i].minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
      self.rotationParts[i].minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
      self.rotationParts[i].minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
      x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partnamei .. "#maxRot"));
      self.rotationParts[i].maxRot = {};
      self.rotationParts[i].maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
      self.rotationParts[i].maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
      self.rotationParts[i].maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
      self.rotationParts[i].rotTime = Utils.getNoNil(getXMLString(xmlFile,  partnamei .. "#rotSpeed"), 2)*1000;
      self.rotationParts[i].touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, partnamei .. "#touchRotLimit"), 10));
    end;

	local worksound = Utils.getFilename("work.wav", self.baseDirectory);
    if worksound ~= nil and worksound ~= "" then
        self.worksound = createSample("work");
        loadSample(self.worksound, worksound, false);
    end;
	
	local RotCheck1Node = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.RotCheck1#index"));
    if RotCheck1Node ~= nil then
        self.RotCheck1 = {};
        self.RotCheck1.node = RotCheck1Node;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.RotCheck1#minRot"));
        self.RotCheck1.minRot = {};
        self.RotCheck1.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.RotCheck1.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.RotCheck1.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.RotCheck1#maxRot"));
        self.RotCheck1.maxRot = {};
        self.RotCheck1.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.RotCheck1.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.RotCheck1.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
        self.RotCheck1.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.RotCheck1#rotTime"), 2)*1000;
    end;
	
	self.HastelRotate = false;
	self.ausklapp = false;
	self.turnon = false; 
	self.rdy = false; 
	self.Warn_hide = false;
	self.Warn_hide_go = false;	

    self.setReelSpeed = SpecializationUtil.callSpecializationsFunction("setReelSpeed");
    self.onStartReel = SpecializationUtil.callSpecializationsFunction("onStartReel");
    self.onStopReel = SpecializationUtil.callSpecializationsFunction("onStopReel");
    self.isReelStarted = Cutter.isReelStarted;
    self.resetFruitType = SpecializationUtil.callSpecializationsFunction("resetFruitType");
    self.setFruitType = SpecializationUtil.callSpecializationsFunction("setFruitType");

    self.reelNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.reel#index"));

    self.rollNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.roll#index"));

    local indexSpikesStr = getXMLString(xmlFile, "vehicle.reelspikes#index");
    self.spikesCount = getXMLInt(xmlFile, "vehicle.reelspikes#count");
    self.spikesRootNode = Utils.indexToObject(self.components, indexSpikesStr);

    self.sideArm = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.sidearms#index"));
    self.sideArmMovable = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.sidearms#movable"), false);

    self.threshingParticleSystems = {};
    local psName = "vehicle.threshingParticleSystem";
    Utils.loadParticleSystem(xmlFile, self.threshingParticleSystems, psName, self.components, false, nil, self.baseDirectory)

    self.fruitExtraObjects = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.fruitExtraObjects.fruitExtraObject(%d)", i);
        local t = getXMLString(xmlFile, key.."#fruitType");
        local index = getXMLString(xmlFile, key.."#index");
        if t==nil or index==nil then
            break;
        end;

        local node = Utils.indexToObject(self.components, index);
        if node ~= nil then
            if self.currentExtraObject == nil then
                self.currentExtraObject = node;
                setVisibility(node, true);
            else
                setVisibility(node, false);
            end;
            self.fruitExtraObjects[t] = node;
        end;
        i = i +1;
    end;

    self.preferedCombineSize = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.preferedCombineSize"), 1);

    self.fruitTypes = {};
    self.fruitTypes[FruitUtil.FRUITTYPE_UNKNOWN] = true;
    local fruitTypes = getXMLString(xmlFile, "vehicle.fruitTypes#fruitTypes");
    if fruitTypes ~= nil then
        local types = Utils.splitString(" ", fruitTypes);
        for k,v in pairs(types) do
            local desc = FruitUtil.fruitTypes[v];
            if desc ~= nil then
                self.fruitTypes[desc.index] = true;
            end;
        end;
    end;

    self.currentFruitType = FruitUtil.FRUITTYPE_UNKNOWN;

    self.reelStarted = false;

    self.forceLowSpeed = false;
    self.speedLimitLow = 12;
    self.speedLimit = 17.5;

    self.speedViolationMaxTime = 50;
    self.speedViolationTimer = self.speedViolationMaxTime;
    self.printRainWarning = false;


    self.lastArea = 0;

end;

function ClaasC540:delete()
    Utils.deleteParticleSystem(self.threshingParticleSystems);
end;

function ClaasC540:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ClaasC540:keyEvent(unicode, sym, modifier, isDown)
		if not Input.isKeyPressed(Input.KEY_x) and self.keystate_6 and self.turnon ~= true  then
			self.keystate_6 = false;
		end;
		
		if Input.isKeyPressed(Input.KEY_x) and not self.keystate_6 and self.turnon ~= true then
			self.keystate_6 = true;
			self.ausklapp = not self.ausklapp;
			if self.Warn_hide == false then
			   self.Warn_hide_go = true; 
			else
				self.Warn_hide_go = false; 
			end;      
		end; 
		
		if not Input.isKeyPressed(Input.KEY_g) and self.keystate_g and self.ausklapp ~= false and self.rdy == true then
			self.keystate_g = false;
		end;
		
		if Input.isKeyPressed(Input.KEY_g) and not self.keystate_g and self.ausklapp ~= false and self.rdy == true then
			self.keystate_g = true;
			self.turnon = not self.turnon;
			self.worksoundRunning = not self.worksoundRunning;
			if self.turnon ~= true then
				Utils.setEmittingState(self.threshingParticleSystems, false);
				stopSample(self.worksound);
				self.worksoundRunning = false;
			end;
			if self.worksoundRunning ~= false then
				playSample(self.worksound, 0, 1, 0);
				self.worksoundRunning = true;
					else
				stopSample(self.worksound);
				self.worksoundRunning = false;
			end;
		end;
end;

function ClaasC540:update(dt)
    self.lastArea = 0;
    if self.reelStarted and self.movingDirection < 0 then

        local speedLimit = self.speedLimit;
        if Cutter.getUseLowSpeedLimit(self) then
            speedLimit = self.speedLimitLow;
        end;

        if self:doCheckSpeedLimit() and self.lastSpeed*3600 > speedLimit then
            self.speedViolationTimer = self.speedViolationTimer - dt;
        else
            self.speedViolationTimer = self.speedViolationMaxTime;
        end;

        if self.speedViolationTimer > 0 then
            if Cutter.allowThreshing(false) then
                self.printRainWarning = false;

                -- during low fill level new fruit types are allowed to "take over"
                local lowFillLevel = false;
                if self.attacherVehicle ~= nil then
                    if self.attacherVehicle.grainTankFillLevel > 0 and self.attacherVehicle.grainTankFillLevel / self.attacherVehicle.grainTankCapacity <= self.attacherVehicle.minThreshold then
                        lowFillLevel = true;
                    end;
                end;

                local foundFruitType = false;
                local oldFruitType = self.currentFruitType;
                
                local allowsThreshing = true;
                if self.attacherVehicle ~= nil then
                    allowsThreshing = self.attacherVehicle.allowsThreshing;
                end;

                if allowsThreshing then
                    if self.currentFruitType == FruitUtil.FRUITTYPE_UNKNOWN or lowFillLevel then
                        for fruitType,v in pairs(self.fruitTypes) do
                            local isOk = true;
                            if self.attacherVehicle ~= nil then
                                if self.attacherVehicle.allowGrainTankFruitType ~= nil then
                                    isOk = self.attacherVehicle:allowGrainTankFruitType(fruitType);
                                end;
                            end;
                            if isOk then
                                for k,area in pairs(self.cuttingAreas) do
                                    local x,y,z = getWorldTranslation(area.start);
                                    local x1,y1,z1 = getWorldTranslation(area.width);
                                    local x2,y2,z2 = getWorldTranslation(area.height);
                                    local area = Utils.getFruitArea(fruitType, x, z, x1, z1, x2, z2);
                                    if area > 0 then
                                        self.currentFruitType = fruitType;
                                        if self.currentFruitType ~= oldFruitType then
                                            Cutter.updateExtraObjects(self)
                                            if self.attacherVehicle ~= nil then
                                                self.attacherVehicle:emptyGrainTankIfLowFillLevel();
                                            end;
                                        end;
                                        foundFruitType = true;
                                        break;
                                    end;
                                end;

                                if foundFruitType then
                                    break;
                                end;
                            end;
                        end;
                    end;
                    if self.currentFruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
                        local realArea = 0;
                        for k,area in pairs(self.cuttingAreas) do
                            local x,y,z = getWorldTranslation(area.start);
                            local x1,y1,z1 = getWorldTranslation(area.width);
                            local x2,y2,z2 = getWorldTranslation(area.height);
                            Utils.updateFruitCutShortArea(self.currentFruitType, x, z, x1, z1, x2, z2, 1);
                            local area = Utils.cutFruitArea(self.currentFruitType, x, z, x1, z1, x2, z2);
                            if area > 0 then
                                local spray = Utils.getDensity(g_currentMission.terrainDetailId, g_currentMission.sprayChannel, x, z, x1, z1, x2, z2);
                                local multi = 1;
                                if spray > 0 then
                                    multi = 2;
                                end;
                                self.lastArea = self.lastArea + area*multi;
                                realArea = realArea + area/g_currentMission.maxFruitValue;
                            end;
                        end;
                        local pixelToQm = 2048 / 4096 * 2048 / 4096; -- 4096px are mapped to 2048m
                        local qm = realArea*pixelToQm;
                        local ha = qm/10000;

                        g_currentMission.missionStats.hectaresThreshedTotal = g_currentMission.missionStats.hectaresThreshedTotal + ha;
                        g_currentMission.missionStats.hectaresThreshedSession = g_currentMission.missionStats.hectaresThreshedSession + ha;

                        g_currentMission.missionStats.threshingDurationTotal = g_currentMission.missionStats.threshingDurationTotal + dt/(1000*60);
                        g_currentMission.missionStats.threshingDurationSession= g_currentMission.missionStats.threshingDurationSession + dt/(1000*60);
                    end;
                end;
            else
                self.printRainWarning = true;
            end;
        end;
    else
        self.speedViolationTimer = self.speedViolationMaxTime;
    end;

    Utils.setEmittingState(self.threshingParticleSystems, (self.reelStarted and self.lastArea > 0.0));
	
			local x, y, z = getRotation(self.RotCheck1.node);
			local minRot = self.RotCheck1.maxRot;
			if math.abs(y-minRot[2]) < 0.005 then
				self.rdy = true; 
				self.Warn_hide_go = false;
			end;
					
			local x, y, z = getRotation(self.RotCheck1.node);
			local minRot = self.RotCheck1.minRot;
			if math.abs(y-minRot[2]) < 0.005 then
				self.rdy = false; 
			end;
		
		if self.ausklapp ~= false then
			for i=1, table.getn(self.rotationParts) do
				local rot = {getRotation(self.rotationParts[i].index)};
				local newRot = Utils.getMovedLimitedValues(rot, self.rotationParts[i].maxRot, self.rotationParts[i].minRot, 3, self.rotationParts[i].rotTime, dt, not self.ausklapp);
				setRotation(self.rotationParts[i].index, unpack(newRot));
			end; 
			
			local x, y, z = getRotation(self.RotCheck1.node);
			local rot = {x,y,z};
			local newRot = Utils.getMovedLimitedValues(rot, self.RotCheck1.maxRot, self.RotCheck1.minRot, 3, self.RotCheck1.rotTime, dt, not self.ausklapp);
			setRotation(self.RotCheck1.node, unpack(newRot));
					
	
		else
				Utils.setEmittingState(self.threshingParticleSystems, false);
				stopSample(self.worksound);
			  
				self.worksoundRunning = false;
				self.turnon = false; 	
				
				local x, y, z = getTranslation(self.translationPart.node);
				local trans = {x,y,z};
				local newTrans = Utils.getMovedLimitedValues(trans, self.translationPart.maxTrans, self.translationPart.minTrans, 3, self.translationPart.transTime, dt, not self.ausklapp);
				setTranslation(self.translationPart.node, unpack(newTrans));
					local x, y, z = getTranslation(self.translationPart2.node);
				local trans = {x,y,z};
				local newTrans = Utils.getMovedLimitedValues(trans, self.translationPart2.maxTrans, self.translationPart2.minTrans, 3, self.translationPart2.transTime, dt, not self.ausklapp);
				setTranslation(self.translationPart2.node, unpack(newTrans));
				
			for i=1, table.getn(self.rotationParts) do
				local rot = {getRotation(self.rotationParts[i].index)};
				local newRot = Utils.getMovedLimitedValues(rot, self.rotationParts[i].maxRot, self.rotationParts[i].minRot, 3, self.rotationParts[i].rotTime, dt, not self.ausklapp);
				setRotation(self.rotationParts[i].index, unpack(newRot));
			end; 
			for i=1, table.getn(self.HaspelParts) do
				local x, y, z = getRotation(self.HaspelParts[i].index);
				local rot = {x,y,z};
				local newRot = Utils.getMovedLimitedValues(rot, self.HaspelParts[i].maxRot, self.HaspelParts[i].minRot, 3, self.HaspelParts[i].rotTime, dt, not self.ausklapp);		
				setRotation(self.HaspelParts[i].index, unpack(newRot));
			end; 
				local x, y, z = getRotation(self.RotCheck1.node);
				local rot = {x,y,z};
				local newRot = Utils.getMovedLimitedValues(rot, self.RotCheck1.maxRot, self.RotCheck1.minRot, 3, self.RotCheck1.rotTime, dt, not self.ausklapp);
				setRotation(self.RotCheck1.node, unpack(newRot));
		end; 
		
		if self.turnon ~= false then
			for i=1, table.getn(self.SpinnerParts) do
				rotate(self.SpinnerParts[i].index , self.SpinnerParts[i].Scale / 1000 * dt ,0,0);
			end;		
		end;
		
		if self.rdy == true then 
			if Input.isKeyPressed(Input.KEY_KP_2) or Input.isKeyPressed(Input.KEY_KP_8)  then
				for i=1, table.getn(self.HaspelParts) do
					local x, y, z = getRotation(self.HaspelParts[i].index);
					local rot = {x,y,z};
					local newRot = Utils.getMovedLimitedValues(rot, self.HaspelParts[i].maxRot, self.HaspelParts[i].minRot, 3, self.HaspelParts[i].rotTime, dt, Input.isKeyPressed(Input.KEY_KP_2));
					setRotation(self.HaspelParts[i].index, unpack(newRot));
				end; 
			end;
		end; 
		
		if self.rdy == true then 
			if Input.isKeyPressed(Input.KEY_KP_1) or Input.isKeyPressed(Input.KEY_KP_7)  then
				local x, y, z = getTranslation(self.translationPart.node);
				local trans = {x,y,z};
				local newTrans = Utils.getMovedLimitedValues(trans, self.translationPart.maxTrans, self.translationPart.minTrans, 3, self.translationPart.transTime, dt, Input.isKeyPressed(Input.KEY_KP_1));
				setTranslation(self.translationPart.node, unpack(newTrans));
					local x, y, z = getTranslation(self.translationPart2.node);
				local trans = {x,y,z};
				local newTrans = Utils.getMovedLimitedValues(trans, self.translationPart2.maxTrans, self.translationPart2.minTrans, 3, self.translationPart2.transTime, dt, Input.isKeyPressed(Input.KEY_KP_1));
				setTranslation(self.translationPart2.node, unpack(newTrans));
			end;
		end; 
		
		if self.rdy == false then
			for i=1, self.numhideWarn do
				local hideWarn = self.hideWarn[i];
				setVisibility(hideWarn, true);
			end; 	
		else
			for i=1, self.numhideWarn do
				local hideWarn = self.hideWarn[i];
				setVisibility(hideWarn, false);
			end; 
		end; 
		
		if self.Warn_hide_go == true then
			for i=1, self.numhideWarn do
				local hideWarn = self.hideWarn[i];
				setVisibility(hideWarn, false);
			end; 	
		end;	

    if self.reelStarted then

        rotate(self.rollNode, -dt*self.reelSpeed*3, 0, 0);

        if self.reelNode ~= nil then
            rotate(self.reelNode, -dt*self.reelSpeed, 0, 0);

            if self.sideArmMovable then
                --:TODO: move arm downwards
            end;

            --correct spikes, so that they always look down
            atx, aty, atz = getRotation(self.reelNode);

            for i=1, self.spikesCount do
                local spike    =    getChildAt(self.spikesRootNode, i-1);
                tx, ty, tz = getRotation(spike);
                setRotation(spike, -atx, aty, atz);
            end;
        end;

    end;

end;

function ClaasC540:draw()

		if self.ausklapp == false then
			renderText(0.72, 0.605, 0.02, "");
		else
			if self.rdy == false then
				renderText(0.72, 0.605, 0.02, "");
			else
				if self.rdy == true then
					if  self.turnon ~= true then
						renderText(0.72, 0.605, 0.02, "");
					else
						renderText(0.72, 0.59, 0.02, "");
					end;
				end; 
			end; 
		end;

    if math.abs(self.speedViolationTimer - self.speedViolationMaxTime) > 2 then
        local str = "2";
        local keyStr = InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)
        if Cutter.getUseLowSpeedLimit(self) then
            str = "1";
            keyStr = InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL1)
        end;
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), str, keyStr), 0.07+0.022, 0.019+0.029);
    end;

    if self.printRainWarning then
        --g_currentMission:addWarning("Dreschen Sie nicht bei Regen oder Hagel", 0.018, 0.033);
        g_currentMission:addWarning(g_i18n:getText("Dont_do_threshing_during_rain_or_hail"), 0.018, 0.033);
    end;

end;

function ClaasC540:onDetach()
    if self.deactivateOnDetach then
        Cutter.onDeactivate(self);
    end;
	self.worksoundRunning = false;	
end;

function ClaasC540:onLeave()
    if self.deactivateOnLeave then
        Cutter.onDeactivate(self);
    end;
end;

function ClaasC540:onDeactivate()
    self:onStopReel();
    Utils.setEmittingState(self.threshingParticleSystems, false);
    self.speedViolationTimer = self.speedViolationMaxTime;
end;

function ClaasC540:setReelSpeed(speed)
    self.reelSpeed = speed;
end;

function ClaasC540:onStartReel()
    self.reelStarted = true;
end;

function ClaasC540:onStopReel()
    self.reelStarted = false;
    Utils.setEmittingState(self.threshingParticleSystems, false);
    self.speedViolationTimer = self.speedViolationMaxTime;
end;

function ClaasC540:isReelStarted()
    return self.reelStarted;
end;

function ClaasC540:resetFruitType()
    self.currentFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
    self.lastArea = 0;
end;

function ClaasC540:setFruitType(fruitType)
    if self.currentFruitType ~= fruitType then
        self.currentFruitType = fruitType;
        self.lastArea = 0;

        ClaasC540.updateExtraObjects(self)
    end;
end;

function ClaasC540.getUseLowSpeedLimit(self)
    if self.forceLowSpeed or (self.attacherVehicle ~= nil and self.preferedCombineSize > self.attacherVehicle.combineSize) then
        return true;
    end;
    return false;
end;

function ClaasC540.updateExtraObjects(self)
    if self.currentExtraObject ~= nil then
        setVisibility(self.currentExtraObject, false);
        self.currentExtraObject = nil;
    end;
    if self.currentFruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
        local name = FruitUtil.fruitIndexToDesc[self.currentFruitType].name;
        local extraObject = self.fruitExtraObjects[name];
        if extraObject ~= nil then
            setVisibility(extraObject, true);
            self.currentExtraObject = extraObject;
        end;
    end;
end;

function ClaasC540.allowThreshing(earlyWarning)
    if early ~= nil and early == true then
        if g_currentMission.environment.lastRainScale <= 0.02 and g_currentMission.environment.timeSinceLastRain > 20 then
            return true;
        end;
    else
        if g_currentMission.environment.lastRainScale <= 0.1 and g_currentMission.environment.timeSinceLastRain > 20 then
            return true;
        end;
    end;
    return false;
end;
